
# q-expansion of newform 912.2.bn.h, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 912
weight = 2
poly_data = [4, 0, -2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 0, 0, 1], 2]]

hecke_ring_character_values = [[799, [1, 0, 0, 0]], [229, [1, 0, 0, 0]], [305, [-1, 0, 0, 0]], [97, [0, 0, 1, 0]]]
aps_data = [[0, 0, 0, 0], [0, 1, -1, -1], [0, 1, 0, 0], [-2, -2, 0, 1], [1, 0, -2, 1], [-4, 0, 2, 0], [-2, -2, -2, 0], [0, 3, 1, -3], [0, -5, 0, 5], [0, -1, 0, -1], [0, 0, 0, -3], [-2, 0, 4, -3], [-3, 0, 3, 0], [-4, 2, 4, -4], [-4, 7, 2, -7], [0, 2, -6, 2], [-9, 1, 9, -2], [0, 1, 4, 1], [2, 3, -1, -3], [-6, 0, 6, 0], [1, 4, -1, -8], [0, -6, 0, 0], [9, 0, -18, 1], [0, 2, 12, 2], [-3, -6, -3, 0], [0, -7, 0, 7], [-2, 0, 4, 6], [0, -4, 0, 2], [0, 0, 0, 0], [-9, 8, 0, -4], [12, 0, -6, 0], [-7, -7, -7, 0], [-2, -4, 1, 4], [0, 3, -7, 3], [2, 1, 2, 0], [8, 0, -16, -3], [4, 6, -4, -12], [-11, 6, 0, -3], [0, -2, 0, -2], [6, 2, -6, -4], [15, 6, 0, -3], [20, -3, -10, 3], [-8, 0, 16, 4], [-10, 6, -10, 0], [-6, 0, 12, -7], [0, 2, 2, 2], [-4, -6, -4, 0], [8, -3, 8, 0], [-3, -2, 0, 1], [-4, -4, 0, 2], [-7, 4, -7, 0], [4, 0, -8, 10], [2, 6, -1, -6], [6, 1, -3, -1], [0, 0, -15, 0], [-6, -1, -6, 0], [0, -10, 0, 20], [2, 9, -2, -18], [8, -10, 0, 5], [0, -8, 3, -8], [-7, 1, 7, -2], [-12, 6, 0, -3], [-5, 9, -5, 0], [0, 0, 0, -11], [0, 0, 19, 0], [0, 0, -6, 0], [-11, 0, 22, -3], [-7, 6, -7, 0], [-3, -1, -3, 0], [8, -20, 0, 10], [3, 0, -6, 14], [0, 17, 0, 0], [0, -3, -16, -3], [0, 0, 0, -18], [4, 0, -8, -6], [-18, 1, 18, -2], [-24, -4, 12, 4], [-2, -3, 2, 6], [3, 4, -3, -8], [14, 6, -7, -6], [6, 0, -12, 10], [-10, -9, -10, 0], [0, -6, -18, -6], [-4, 12, 2, -12], [4, -3, 4, 0], [-6, -17, 3, 17], [21, -8, 0, 4], [-13, 0, 0, 0], [-14, 10, -14, 0], [10, -8, 0, 4], [-13, 0, 26, 7], [-24, -14, 12, 14], [-12, 0, 24, -3], [6, -4, 6, 0], [5, -1, -5, 2], [0, -11, 0, 11], [0, -12, -12, -12], [-21, 4, 0, -2], [-12, -12, 6, 12], [0, 6, 4, 6], [4, -24, -2, 24], [-16, -10, 8, 10], [-9, 26, 0, -13], [0, 28, 0, -14], [-11, 18, 0, -9], [-1, -16, 0, 8], [6, -4, 6, 0], [-6, -16, 3, 16], [0, -7, -12, -7], [-13, 0, 26, 6], [-14, 0, 28, 9], [-38, 4, 38, -8], [14, 2, -7, -2], [16, -12, 0, 6], [0, 5, -22, 5], [3, 12, -3, -24], [-7, -7, 7, 14], [-2, 0, 4, -8], [8, 0, -16, -4], [0, 0, 24, 0], [0, -21, 0, 21], [2, 0, -4, 0], [-18, 12, 0, -6], [12, 24, 0, -12], [40, 0, 0, 0], [0, 7, 0, 0], [0, -3, 10, -3], [-6, 14, -6, 0], [8, 0, -8, 0], [2, 18, 0, -9], [17, -3, -17, 6], [12, 3, -12, -6], [-4, 12, 2, -12], [10, -12, -10, 24], [-3, 0, 6, 26], [0, 8, 16, 8], [0, 1, 0, 1], [3, 0, -6, -9], [30, -6, 0, 3], [3, 0, -6, 2], [-4, 0, 2, 0], [-4, 2, 2, -2], [0, 6, -34, 6], [0, -5, 9, -5], [16, 0, -32, -6], [-6, 3, 6, -6], [10, 0, -10, 0], [-3, -8, 3, 16], [0, -13, -13, -13], [-24, -6, 0, 3], [-12, 15, -12, 0], [19, 0, -38, -16], [0, 13, 11, 13], [0, 18, -12, 18], [-5, -33, -5, 0], [36, 6, 0, -3], [22, -12, 0, 6], [-3, 16, -3, 0], [0, 0, -23, 0], [0, -20, -6, -20], [6, 2, 6, 0], [-15, 4, 15, -8], [-22, 12, 22, -24], [-9, -7, 9, 14], [45, 4, 0, -2], [48, -4, -48, 8], [-20, -12, -20, 0], [0, -21, -8, -21]]
